@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2006 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 2.4 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -892,20 +892,20 @@ operator , ( const sc_logic& a, sc_bitref_r<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_bitref_r<T>,sc_lv_base>
+sc_concref_r<sc_bitref_r<T>,sc_bv_base>
 operator , ( sc_bitref_r<T> a, bool b )
 {
-    return sc_concref_r<sc_bitref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_bitref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_bitref_r<T> >
+sc_concref_r<sc_bv_base,sc_bitref_r<T> >
 operator , ( bool a, sc_bitref_r<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_bitref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_bitref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -947,20 +947,20 @@ concat( const sc_logic& a, sc_bitref_r<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_bitref_r<T>,sc_lv_base>
+sc_concref_r<sc_bitref_r<T>,sc_bv_base>
 concat( sc_bitref_r<T> a, bool b )
 {
-    return sc_concref_r<sc_bitref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_bitref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_bitref_r<T> >
+sc_concref_r<sc_bv_base,sc_bitref_r<T> >
 concat( bool a, sc_bitref_r<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_bitref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_bitref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1004,20 +1004,20 @@ operator , ( const sc_logic& a, sc_bitref<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_bitref_r<T>,sc_lv_base>
+sc_concref_r<sc_bitref_r<T>,sc_bv_base>
 operator , ( sc_bitref<T> a, bool b )
 {
-    return sc_concref_r<sc_bitref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_bitref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_bitref_r<T> >
+sc_concref_r<sc_bv_base,sc_bitref_r<T> >
 operator , ( bool a, sc_bitref<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_bitref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_bitref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1059,20 +1059,20 @@ concat( const sc_logic& a, sc_bitref<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_bitref_r<T>,sc_lv_base>
+sc_concref_r<sc_bitref_r<T>,sc_bv_base>
 concat( sc_bitref<T> a, bool b )
 {
-    return sc_concref_r<sc_bitref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_bitref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_bitref_r<T> >
+sc_concref_r<sc_bv_base,sc_bitref_r<T> >
 concat( bool a, sc_bitref<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_bitref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_bitref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 #endif
@@ -1124,20 +1124,20 @@ operator , ( const sc_logic& a, sc_subref_r<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_subref_r<T>,sc_lv_base>
+sc_concref_r<sc_subref_r<T>,sc_bv_base>
 operator , ( sc_subref_r<T> a, bool b )
 {
-    return sc_concref_r<sc_subref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_subref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_subref_r<T> >
+sc_concref_r<sc_bv_base,sc_subref_r<T> >
 operator , ( bool a, sc_subref_r<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_subref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_subref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1179,20 +1179,20 @@ concat( const sc_logic& a, sc_subref_r<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_subref_r<T>,sc_lv_base>
+sc_concref_r<sc_subref_r<T>,sc_bv_base>
 concat( sc_subref_r<T> a, bool b )
 {
-    return sc_concref_r<sc_subref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_subref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_subref_r<T> >
+sc_concref_r<sc_bv_base,sc_subref_r<T> >
 concat( bool a, sc_subref_r<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_subref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_subref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1236,20 +1236,20 @@ operator , ( const sc_logic& a, sc_subref<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_subref_r<T>,sc_lv_base>
+sc_concref_r<sc_subref_r<T>,sc_bv_base>
 operator , ( sc_subref<T> a, bool b )
 {
-    return sc_concref_r<sc_subref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_subref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_subref_r<T> >
+sc_concref_r<sc_bv_base,sc_subref_r<T> >
 operator , ( bool a, sc_subref<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_subref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_subref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1291,20 +1291,20 @@ concat( const sc_logic& a, sc_subref<T> b )
 
 template <class T>
 inline
-sc_concref_r<sc_subref_r<T>,sc_lv_base>
+sc_concref_r<sc_subref_r<T>,sc_bv_base>
 concat( sc_subref<T> a, bool b )
 {
-    return sc_concref_r<sc_subref_r<T>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_subref_r<T>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,sc_subref_r<T> >
+sc_concref_r<sc_bv_base,sc_subref_r<T> >
 concat( bool a, sc_subref<T> b )
 {
-    return sc_concref_r<sc_lv_base,sc_subref_r<T> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_subref_r<T> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 #endif
@@ -1401,20 +1401,20 @@ operator , ( const sc_logic& a, sc_concref_r<T1,T2> b )
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
 operator , ( sc_concref_r<T1,T2> a, bool b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
 operator , ( bool a, sc_concref_r<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1423,8 +1423,8 @@ inline
 sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
 concat( sc_concref_r<T1,T2> a, const char* b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( b ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+        ( *a.clone(), *new sc_lv_base( b ), 3 );
 }
 
 template <class T1, class T2>
@@ -1432,8 +1432,8 @@ inline
 sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
 concat( const char* a, sc_concref_r<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( a ), *b.clone(), 3 );
+    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+        ( *new sc_lv_base( a ), *b.clone(), 3 );
 }
 
 template <class T1, class T2>
@@ -1441,8 +1441,8 @@ inline
 sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
 concat( sc_concref_r<T1,T2> a, const sc_logic& b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( b, 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+        ( *a.clone(), *new sc_lv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
@@ -1450,26 +1450,26 @@ inline
 sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
 concat( const sc_logic& a, sc_concref_r<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( a, 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+        ( *new sc_lv_base( a, 1 ), *b.clone(), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
 concat( sc_concref_r<T1,T2> a, bool b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
 concat( bool a, sc_concref_r<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1480,8 +1480,8 @@ inline
 sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
 operator , ( sc_concref<T1,T2> a, const char* b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( b ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+        ( *a.clone(), *new sc_lv_base( b ), 3 );
 }
 
 template <class T1, class T2>
@@ -1489,8 +1489,8 @@ inline
 sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
 operator , ( const char* a, sc_concref<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( a ), *b.clone(), 3 );
+    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+        ( *new sc_lv_base( a ), *b.clone(), 3 );
 }
 
 template <class T1, class T2>
@@ -1498,8 +1498,8 @@ inline
 sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
 operator , ( sc_concref<T1,T2> a, const sc_logic& b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( b, 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+        ( *a.clone(), *new sc_lv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
@@ -1507,26 +1507,26 @@ inline
 sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
 operator , ( const sc_logic& a, sc_concref<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( a, 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+        ( *new sc_lv_base( a, 1 ), *b.clone(), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
 operator , ( sc_concref<T1,T2> a, bool b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
 operator , ( bool a, sc_concref<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 
@@ -1535,8 +1535,8 @@ inline
 sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
 concat( sc_concref<T1,T2> a, const char* b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( b ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+        ( *a.clone(), *new sc_lv_base( b ), 3 );
 }
 
 template <class T1, class T2>
@@ -1544,8 +1544,8 @@ inline
 sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
 concat( const char* a, sc_concref<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( a ), *b.clone(), 3 );
+    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+        ( *new sc_lv_base( a ), *b.clone(), 3 );
 }
 
 template <class T1, class T2>
@@ -1553,8 +1553,8 @@ inline
 sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
 concat( sc_concref<T1,T2> a, const sc_logic& b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( b, 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+        ( *a.clone(), *new sc_lv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
@@ -1562,26 +1562,26 @@ inline
 sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
 concat( const sc_logic& a, sc_concref<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( a, 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+        ( *new sc_lv_base( a, 1 ), *b.clone(), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>
+sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
 concat( sc_concref<T1,T2> a, bool b )
 {
-    return sc_concref_r<sc_concref_r<T1,T2>,sc_lv_base>(
-	*a.clone(), *new sc_lv_base( sc_logic( b ), 1 ), 3 );
+    return sc_concref_r<sc_concref_r<T1,T2>,sc_bv_base>
+        ( *a.clone(), *new sc_bv_base( b, 1 ), 3 );
 }
 
 template <class T1, class T2>
 inline
-sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >
+sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
 concat( bool a, sc_concref<T1,T2> b )
 {
-    return sc_concref_r<sc_lv_base,sc_concref_r<T1,T2> >(
-	*new sc_lv_base( sc_logic( a ), 1 ), *b.clone(), 3 );
+    return sc_concref_r<sc_bv_base,sc_concref_r<T1,T2> >
+        ( *new sc_bv_base( a, 1 ), *b.clone(), 3 );
 }
 
 #endif
@@ -1601,8 +1601,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 operator , ( const sc_proxy<T>& a, const char* b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+      ( a.back_cast(), *new sc_lv_base( b ), 2 );
 }
 
 template <class T>
@@ -1610,8 +1610,8 @@ inline
 sc_concref_r<sc_lv_base,T>
 operator , ( const char* a, const sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+      ( *new sc_lv_base( a ), b.back_cast(), 1 );
 }
 
 template <class T>
@@ -1619,8 +1619,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 operator , ( const sc_proxy<T>& a, const sc_logic& b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+      ( a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
 }
 
 template <class T>
@@ -1628,26 +1628,26 @@ inline
 sc_concref_r<sc_lv_base,T>
 operator , ( const sc_logic& a, const sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a, 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+      ( *new sc_lv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 template <class T>
 inline
-sc_concref_r<T,sc_lv_base>
+sc_concref_r<T,sc_bv_base>
 operator , ( const sc_proxy<T>& a, bool b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( sc_logic( b ), 1 ), 2 );
+    return sc_concref_r<T,sc_bv_base>
+        ( a.back_cast(), *new sc_bv_base( b, 1 ), 2 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,T>
+sc_concref_r<sc_bv_base,T>
 operator , ( bool a, const sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( sc_logic( a ), 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_bv_base,T>
+      ( *new sc_bv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 
@@ -1656,8 +1656,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 concat( const sc_proxy<T>& a, const char* b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+      ( a.back_cast(), *new sc_lv_base( b ), 2 );
 }
 
 template <class T>
@@ -1665,8 +1665,8 @@ inline
 sc_concref_r<sc_lv_base,T>
 concat( const char* a, const sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+      ( *new sc_lv_base( a ), b.back_cast(), 1 );
 }
 
 template <class T>
@@ -1674,8 +1674,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 concat( const sc_proxy<T>& a, const sc_logic& b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+      ( a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
 }
 
 template <class T>
@@ -1683,26 +1683,26 @@ inline
 sc_concref_r<sc_lv_base,T>
 concat( const sc_logic& a, const sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a, 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+      ( *new sc_lv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 template <class T>
 inline
-sc_concref_r<T,sc_lv_base>
+sc_concref_r<T,sc_bv_base>
 concat( const sc_proxy<T>& a, bool b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( sc_logic( b ), 1 ), 2 );
+    return sc_concref_r<T,sc_bv_base>
+      ( a.back_cast(), *new sc_bv_base( b, 1 ), 2 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,T>
+sc_concref_r<sc_bv_base,T>
 concat( bool a, const sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( sc_logic( a ), 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_bv_base,T>
+      ( *new sc_bv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 
@@ -1713,8 +1713,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 operator , ( sc_proxy<T>& a, const char* b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+      ( a.back_cast(), *new sc_lv_base( b ), 2 );
 }
 
 template <class T>
@@ -1722,8 +1722,8 @@ inline
 sc_concref_r<sc_lv_base,T>
 operator , ( const char* a, sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+        ( *new sc_lv_base( a ), b.back_cast(), 1 );
 }
 
 template <class T>
@@ -1731,8 +1731,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 operator , ( sc_proxy<T>& a, const sc_logic& b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+        ( a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
 }
 
 template <class T>
@@ -1740,26 +1740,26 @@ inline
 sc_concref_r<sc_lv_base,T>
 operator , ( const sc_logic& a, sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a, 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+        ( *new sc_lv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 template <class T>
 inline
-sc_concref_r<T,sc_lv_base>
+sc_concref_r<T,sc_bv_base>
 operator , ( sc_proxy<T>& a, bool b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( sc_logic( b ), 1 ), 2 );
+    return sc_concref_r<T,sc_bv_base>
+        ( a.back_cast(), *new sc_bv_base( b, 1 ), 2 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,T>
+sc_concref_r<sc_bv_base,T>
 operator , ( bool a, sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( sc_logic( a ), 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_bv_base,T>
+        ( *new sc_bv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 
@@ -1768,8 +1768,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 concat( sc_proxy<T>& a, const char* b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+        ( a.back_cast(), *new sc_lv_base( b ), 2 );
 }
 
 template <class T>
@@ -1777,8 +1777,8 @@ inline
 sc_concref_r<sc_lv_base,T>
 concat( const char* a, sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+        ( *new sc_lv_base( a ), b.back_cast(), 1 );
 }
 
 template <class T>
@@ -1786,8 +1786,8 @@ inline
 sc_concref_r<T,sc_lv_base>
 concat( sc_proxy<T>& a, const sc_logic& b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
+    return sc_concref_r<T,sc_lv_base>
+        ( a.back_cast(), *new sc_lv_base( b, 1 ), 2 );
 }
 
 template <class T>
@@ -1795,26 +1795,26 @@ inline
 sc_concref_r<sc_lv_base,T>
 concat( const sc_logic& a, sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( a, 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_lv_base,T>
+        ( *new sc_lv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 template <class T>
 inline
-sc_concref_r<T,sc_lv_base>
+sc_concref_r<T,sc_bv_base>
 concat( sc_proxy<T>& a, bool b )
 {
-    return sc_concref_r<T,sc_lv_base>(
-	a.back_cast(), *new sc_lv_base( sc_logic( b ), 1 ), 2 );
+    return sc_concref_r<T,sc_bv_base>
+        ( a.back_cast(), *new sc_bv_base( b, 1 ), 2 );
 }
 
 template <class T>
 inline
-sc_concref_r<sc_lv_base,T>
+sc_concref_r<sc_bv_base,T>
 concat( bool a, sc_proxy<T>& b )
 {
-    return sc_concref_r<sc_lv_base,T>(
-	*new sc_lv_base( sc_logic( a ), 1 ), b.back_cast(), 1 );
+    return sc_concref_r<sc_bv_base,T>
+        ( *new sc_bv_base( a, 1 ), b.back_cast(), 1 );
 }
 
 #endif
