@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2006 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 2.4 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -107,12 +107,11 @@ public:
     const T*& value_ptr();
 
 private:
-
     static sc_global<T>* m_instance;
 
-    sc_core::sc_phash<const sc_core::sc_process_b*,const T*> m_map;
-    const sc_core::sc_process_b*                             m_proc;
-    const T*                                                    m_value_ptr;
+    sc_core::sc_phash<void*,const T*> m_map;
+    void*                             m_proc; // context (current process or NULL)
+    const T*                          m_value_ptr;
 
 };
 
@@ -173,14 +172,13 @@ private:
 template <class T>
 sc_global<T>* sc_global<T>::m_instance = 0;
 
-
 template <class T>
 inline
 sc_global<T>::sc_global()
-: m_map(),
-  m_proc( 
-	reinterpret_cast<const sc_core::sc_process_b*>( -1 ) ), 
-	m_value_ptr( 0 )
+  : m_map()
+  // use &m_instance as unique "non-process" key (NULL denotes 'sc_main' context)
+  , m_proc( &m_instance )
+  , m_value_ptr( 0 )
 {}
 
 
@@ -189,7 +187,7 @@ inline
 void
 sc_global<T>::update()
 {
-    const sc_core::sc_process_b* p = sc_core::sc_get_current_process_b();
+    void* p = sc_core::sc_get_current_process_b();
     if( p != m_proc )
     {
         const T* vp = m_map[p];
