@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2011 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 3.0 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -592,8 +592,8 @@ and_on_help(small_type us,
             const sc_digit *vd)
 {
 
-  register sc_digit *x = ud;
-  register const sc_digit *y = vd;
+  sc_digit *x = ud;
+  const sc_digit *y = vd;
   int xnd = und;
   int ynd = vnd;
 
@@ -621,8 +621,8 @@ and_on_help(small_type us,
     }
     else {  // case 3
 
-      register sc_digit xcarry = 1;
-      register sc_digit ycarry = 1;
+      sc_digit xcarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -646,7 +646,7 @@ and_on_help(small_type us,
 
     if (us > 0) { // case 4
 
-      register sc_digit ycarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         ycarry += (~(*y++) & DIGIT_MASK);
@@ -663,7 +663,7 @@ and_on_help(small_type us,
     }
     else {  // case 5
 
-      register sc_digit xcarry = 1;
+      sc_digit xcarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -693,8 +693,8 @@ or_on_help(small_type us,
            const sc_digit *vd)
 {
   
-  register sc_digit *x = ud;
-  register const sc_digit *y = vd;
+  sc_digit *x = ud;
+  const sc_digit *y = vd;
   int xnd = und;
   int ynd = vnd;
 
@@ -720,8 +720,8 @@ or_on_help(small_type us,
     }
     else {  // case 4
 
-      register sc_digit xcarry = 1;
-      register sc_digit ycarry = 1;
+      sc_digit xcarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -745,7 +745,7 @@ or_on_help(small_type us,
 
     if (us > 0) { // case 5
 
-      register sc_digit ycarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         ycarry += (~(*y++) & DIGIT_MASK);
@@ -764,7 +764,7 @@ or_on_help(small_type us,
     }
     else {  // case 6
 
-      register sc_digit xcarry = 1;
+      sc_digit xcarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -796,8 +796,8 @@ xor_on_help(small_type us,
             const sc_digit *vd)
 {
   
-  register sc_digit *x = ud;
-  register const sc_digit *y = vd;
+  sc_digit *x = ud;
+  const sc_digit *y = vd;
   int xnd = und;
   int ynd = vnd;
 
@@ -826,8 +826,8 @@ xor_on_help(small_type us,
     }
     else {  // case 4
 
-      register sc_digit xcarry = 1;
-      register sc_digit ycarry = 1;
+      sc_digit xcarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
@@ -850,7 +850,7 @@ xor_on_help(small_type us,
 
     if (us > 0) { // case 5
 
-      register sc_digit ycarry = 1;
+      sc_digit ycarry = 1;
 
       while (y < yend) {
         ycarry += (~(*y++) & DIGIT_MASK);
@@ -869,7 +869,7 @@ xor_on_help(small_type us,
     }
     else {  // case 6
 
-      register sc_digit xcarry = 1;
+      sc_digit xcarry = 1;
 
       while (y < yend) {
         xcarry += (~(*x) & DIGIT_MASK);
