@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2011 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 3.0 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -24,15 +24,15 @@
  CHANGE LOG APPEARS AT THE END OF THE FILE
  *****************************************************************************/
 
-#if !defined(WIN32) && defined(SC_USE_PTHREADS)
+#if !defined(_WIN32) && !defined(WIN32) && defined(SC_USE_PTHREADS)
 
 // ORDER OF THE INCLUDES AND namespace sc_core IS IMPORTANT!!!
 
-using namespace std;
-
 #include "sysc/kernel/sc_cor_pthread.h"
 #include "sysc/kernel/sc_simcontext.h"
 
+using namespace std;
+
 namespace sc_core {
 
 // MAKE SURE WE HAVE A NULL THAT WILL WORK:
@@ -281,7 +281,7 @@ sc_cor_pkg_pthread::get_main()
 
 } // namespace sc_core
 
-#endif // !defined(WIN32) && defined(SC_USE_PTHREADS)
+#endif // !defined(_WIN32) && !defined(WIN32) && defined(SC_USE_PTHREADS)
 
 
 // $Log: sc_cor_pthread.cpp,v $
