@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2011 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 3.0 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -28,16 +28,11 @@
 #ifndef SC_EVENT_H
 #define SC_EVENT_H
 
+#include "sysc/kernel/sc_cmnhdr.h"
 #include "sysc/kernel/sc_kernel_ids.h"
 #include "sysc/kernel/sc_simcontext.h"
 #include "sysc/communication/sc_writer_policy.h"
 
-namespace sc_dt {
-
-class sc_logic; // needed for sc_signal<sc_logic> friend
-
-} // namespace sc_std
-
 namespace sc_core {
 
 // forward declarations
@@ -257,8 +252,6 @@ class sc_event
     friend class sc_method_process;
     friend class sc_thread_process;
     template<typename IF, sc_writer_policy POL> friend class sc_signal;
-    friend class sc_signal<bool>;
-    friend class sc_signal<sc_dt::sc_logic>;
     friend void sc_thread_cor_fn( void* arg );
 
 public:
@@ -707,7 +700,7 @@ sc_event_or_list::swap( sc_event_or_list & that )
 
 inline
 sc_event_and_list::sc_event_and_list()
-  : sc_event_list( false )
+  : sc_event_list( true )
 {}
 
 inline
