@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2006 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 2.4 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -59,9 +59,10 @@
  *****************************************************************************/
 
 
-#include <assert.h>
-#include <time.h>
 #include <cstdlib>
+#include <vector>
+
+#define SC_DISABLE_API_VERSION_CHECK // for in-library sc_ver.h inclusion
 
 #include "sysc/kernel/sc_simcontext.h"
 #include "sysc/kernel/sc_ver.h"
@@ -77,8 +78,6 @@
 
 namespace sc_core {
 
-static bool running_regression = false;
-
 // Forward declarations for functions that come later in the file
 static char map_sc_logic_state_to_wif_state(char in_char);
 
@@ -125,25 +124,27 @@ wif_trace::wif_trace(const std::string& name_,
 {
     /* Intentionally blank */
 }
-        
+
 void
 wif_trace::print_variable_declaration_line( FILE* f )
 {
-    char buf[2000];
-
-    if( bit_width < 0 ) {
-        std::sprintf( buf, "Traced object \"%s\" has < 0 Bits, cannot be traced.",
-	      name.c_str() );
-        put_error_message( buf, false );
-    } else {
-        std::fprintf( f, "declare  %s   \"%s\"  %s  ",
-	    wif_name.c_str(), name.c_str(), wif_type );
-	if( bit_width > 0 ) {
-	    std::fprintf( f, "0 %d ", bit_width - 1 );
-	}
-	std::fprintf( f, "variable ;\n" );
-	std::fprintf( f, "start_trace %s ;\n", wif_name.c_str() );
+    if( bit_width < 0 )
+    {
+        std::stringstream ss;
+        ss << "'" << name << "' has < 0 bits";
+        SC_REPORT_ERROR( SC_ID_TRACING_OBJECT_IGNORED_
+                       , ss.str().c_str() );
+        return;
     }
+
+    std::fprintf( f, "declare  %s   \"%s\"  %s  ",
+                  wif_name.c_str(), name.c_str(), wif_type );
+
+    if( bit_width > 0 ) {
+        std::fprintf( f, "0 %d ", bit_width - 1 );
+    }
+    std::fprintf( f, "variable ;\n" );
+    std::fprintf( f, "start_trace %s ;\n", wif_name.c_str() );
 }
 
 void
@@ -185,7 +186,8 @@ wif_uint64_trace::wif_uint64_trace(const sc_dt::uint64& object_,
   mask(static_cast<sc_dt::uint64>(-1))
 {
     bit_width = width_;
-    if (bit_width < 32) mask = ~(mask << bit_width);
+    if (bit_width < (int)(sizeof(sc_dt::uint64)*BITS_PER_BYTE))
+        mask = ~(mask << bit_width);
     wif_type = "BIT";
 }
 
@@ -250,7 +252,8 @@ wif_int64_trace::wif_int64_trace(const sc_dt::int64& object_,
   mask(static_cast<sc_dt::uint64>(-1))
 {
     bit_width = width_;
-    if (bit_width < 32) mask = ~(mask << bit_width);
+    if (bit_width < (int)(sizeof(sc_dt::int64)*BITS_PER_BYTE))
+        mask = ~(mask << bit_width);
     wif_type = "BIT";
 }
 
@@ -449,14 +452,20 @@ bool wif_sc_unsigned_trace::changed()
 
 void wif_sc_unsigned_trace::write(FILE* f)
 {
-    char buf[1000], *buf_ptr = buf;
+    static std::vector<char> buf(1024);
+    typedef std::vector<char>::size_type size_t;
 
-    int bitindex;
-    for(bitindex = object.length() - 1; bitindex >= 0; --bitindex) {
-        *buf_ptr++ = "01"[(object)[bitindex]];
+    if ( buf.size() < (size_t)object.length() ) {
+        size_t sz = ( (size_t)object.length() + 4096 ) & (~(size_t)(4096-1));
+        std::vector<char>( sz ).swap( buf ); // resize without copying values
+    }
+    char *buf_ptr = &buf[0];
+
+    for(int bitindex = object.length() - 1; bitindex >= 0; --bitindex) {
+        *buf_ptr++ = "01"[object[bitindex].to_bool()];
     }
     *buf_ptr = '\0';
-    std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(), buf); 
+    std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(), &buf[0]);
     old_value = object;
 }
 
@@ -499,15 +508,21 @@ bool wif_sc_signed_trace::changed()
 
 void wif_sc_signed_trace::write(FILE* f)
 {
-    char buf[1000], *buf_ptr = buf;
+    static std::vector<char> buf(1024);
+    typedef std::vector<char>::size_type size_t;
 
-    int bitindex;
-    for(bitindex = object.length() - 1; bitindex >= 0; --bitindex) {
-        *buf_ptr++ = "01"[(object)[bitindex]];
+    if ( buf.size() < (size_t)object.length() ) {
+        size_t sz = ( (size_t)object.length() + 4096 ) & (~(size_t)(4096-1));
+        std::vector<char>( sz ).swap( buf ); // resize without copying values
+    }
+    char *buf_ptr = &buf[0];
+
+    for(int bitindex = object.length() - 1; bitindex >= 0; --bitindex) {
+        *buf_ptr++ = "01"[object[bitindex].to_bool()];
     }
     *buf_ptr = '\0';
 
-    std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(), buf); 
+    std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(), &buf[0]);
     old_value = object;
 }
 
@@ -554,7 +569,7 @@ void wif_sc_uint_base_trace::write(FILE* f)
 
     int bitindex;
     for(bitindex = object.length() - 1; bitindex >= 0; --bitindex) {
-        *buf_ptr++ = "01"[(object)[bitindex]];
+        *buf_ptr++ = "01"[object[bitindex].to_bool()];
     }
     *buf_ptr = '\0';
     std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(), buf); 
@@ -604,7 +619,7 @@ void wif_sc_int_base_trace::write(FILE* f)
 
     int bitindex;
     for(bitindex = object.length() - 1; bitindex >= 0; --bitindex) {
-        *buf_ptr++ = "01"[(object)[bitindex]];
+        *buf_ptr++ = "01"[object[bitindex].to_bool()];
     }
     *buf_ptr = '\0';
 
@@ -744,16 +759,22 @@ wif_sc_fxnum_trace::changed()
 void
 wif_sc_fxnum_trace::write( FILE* f )
 {
-    char buf[1000], *buf_ptr = buf;
+    static std::vector<char> buf(1024);
+    typedef std::vector<char>::size_type size_t;
 
-    int bitindex;
-    for( bitindex = object.wl() - 1; bitindex >= 0; -- bitindex )
+    if ( buf.size() < (size_t)object.wl() ) {
+        size_t sz = ( (size_t)object.wl() + 4096 ) & (~(size_t)(4096-1));
+        std::vector<char>( sz ).swap( buf ); // resize without copying values
+    }
+    char *buf_ptr = &buf[0];
+
+    for(int bitindex = object.wl() - 1; bitindex >= 0; --bitindex)
     {
-        *buf_ptr ++ = "01"[(object)[bitindex]];
+        *buf_ptr ++ = "01"[object[bitindex]];
     }
     *buf_ptr = '\0';
 
-    std::fprintf( f, "assign %s \"%s\" ;\n", wif_name.c_str(), buf );
+    std::fprintf( f, "assign %s \"%s\" ;\n", wif_name.c_str(), &buf[0]);
     old_value = object;
 }
 
@@ -807,16 +828,22 @@ wif_sc_fxnum_fast_trace::changed()
 void
 wif_sc_fxnum_fast_trace::write( FILE* f )
 {
-    char buf[1000], *buf_ptr = buf;
+    static std::vector<char> buf(1024);
+    typedef std::vector<char>::size_type size_t;
 
-    int bitindex;
-    for( bitindex = object.wl() - 1; bitindex >= 0; -- bitindex )
+    if ( buf.size() < (size_t)object.wl() ) {
+        size_t sz = ( (size_t)object.wl() + 4096 ) & (~(size_t)(4096-1));
+        std::vector<char>( sz ).swap( buf ); // resize without copying values
+    }
+    char *buf_ptr = &buf[0];
+
+    for(int bitindex = object.wl() - 1; bitindex >= 0; --bitindex)
     {
-        *buf_ptr ++ = "01"[(object)[bitindex]];
+        *buf_ptr ++ = "01"[object[bitindex]];
     }
     *buf_ptr = '\0';
 
-    std::fprintf( f, "assign %s \"%s\" ;\n", wif_name.c_str(), buf );
+    std::fprintf( f, "assign %s \"%s\" ;\n", wif_name.c_str(), &buf[0]);
     old_value = object;
 }
 
@@ -854,8 +881,6 @@ wif_unsigned_int_trace::wif_unsigned_int_trace(const unsigned& object_,
     bit_width = width_;
     if (bit_width < 32) {
         mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
     }
 
     wif_type = "BIT";
@@ -919,9 +944,7 @@ wif_unsigned_short_trace::wif_unsigned_short_trace(
 {
     bit_width = width_;
     if (bit_width < 16) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffff;
+        mask = (unsigned short)~(-1 << bit_width);
     }
 
     wif_type = "BIT";
@@ -983,9 +1006,7 @@ wif_unsigned_char_trace::wif_unsigned_char_trace(const unsigned char& object_,
 {
     bit_width = width_;
     if (bit_width < 8) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xff;
+        mask = (unsigned char)~(-1 << bit_width);
     }
 
     wif_type = "BIT";
@@ -1044,13 +1065,11 @@ wif_unsigned_long_trace::wif_unsigned_long_trace(const unsigned long& object_,
 					     const std::string& wif_name_,
 					     int width_) 
 : wif_trace(name_, wif_name_), object(object_), old_value(object_),
-  mask(0xffffffff)
+  mask((unsigned long)-1L)
 {
     bit_width = width_;
-    if (bit_width < 32) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
+    if (bit_width < (int)(sizeof(unsigned long)*BITS_PER_BYTE)) {
+        mask = ~(-1L << bit_width);
     }
 
     wif_type = "BIT";
@@ -1075,7 +1094,7 @@ void wif_unsigned_long_trace::write(FILE* f)
         }
     }
     else{
-        unsigned bit_mask = 1 << (bit_width-1);
+        unsigned long bit_mask = 1ul << (bit_width-1);
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             buf[bitindex] = (object & bit_mask)? '1' : '0';
             bit_mask = bit_mask >> 1;
@@ -1114,8 +1133,6 @@ wif_signed_int_trace::wif_signed_int_trace(const signed& object_,
     bit_width = width_;
     if (bit_width < 32) {
         mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
     }
 
     wif_type = "BIT";
@@ -1177,9 +1194,7 @@ wif_signed_short_trace::wif_signed_short_trace(const short& object_,
 {
     bit_width = width_;
     if (bit_width < 16) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffff;
+        mask = (unsigned short)~(-1 << bit_width);
     }
 
     wif_type = "BIT";
@@ -1241,9 +1256,7 @@ wif_signed_char_trace::wif_signed_char_trace(const char& object_,
 {
     bit_width = width_;
     if (bit_width < 8) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xff;
+        mask = (unsigned char)~(-1 << bit_width);
     }
 
     wif_type = "BIT";
@@ -1302,13 +1315,11 @@ wif_signed_long_trace::wif_signed_long_trace(const long& object_,
 					     const std::string& wif_name_,
 					     int width_) 
 : wif_trace(name_, wif_name_), object(object_), old_value(object_),
-  mask(0xffffffff)
+  mask((unsigned long)-1L)
 {
     bit_width = width_;
-    if (bit_width < 32) {
-        mask = ~(-1 << bit_width);
-    } else {
-        mask = 0xffffffff;
+    if (bit_width < (int)(sizeof(long)*BITS_PER_BYTE)) {
+        mask = ~(-1L << bit_width);
     }
 
     wif_type = "BIT";
@@ -1332,7 +1343,7 @@ void wif_signed_long_trace::write(FILE* f)
             buf[bitindex]='0';
         }
     } else {
-        unsigned bit_mask = 1 << (bit_width-1);
+        unsigned long bit_mask = 1ul << (bit_width-1);
         for (bitindex = 0; bitindex < bit_width; bitindex++) {
             buf[bitindex] = (object & bit_mask)? '1' : '0';
             bit_mask = bit_mask >> 1;
@@ -1475,21 +1486,22 @@ bool wif_enum_trace::changed()
 
 void wif_enum_trace::write(FILE* f)
 {
-    char buf[2000];
     static bool warning_issued = false;
+    const char* lit;
 
     if (object >= nliterals) { // Note unsigned value is always greater than 0
         if (!warning_issued) {
-	    std::sprintf(buf, "Tracing error: Value of enumerated type undefined");
-	    put_error_message(buf, false);
-	    warning_issued = true;
-	}
-	std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(),
-		"SC_WIF_UNDEF");
+            SC_REPORT_WARNING( SC_ID_TRACING_INVALID_ENUM_VALUE_ 
+                             , name.c_str() );
+            warning_issued = true;
+        }
+        lit = "SC_WIF_UNDEF";
     }
-    else 
-        std::fprintf(f, "assign %s \"%s\" ;\n", wif_name.c_str(),
-		literals[object]);
+    else
+    {
+        lit = literals[object];
+    }
+    std::fprintf( f, "assign %s \"%s\" ;\n", wif_name.c_str(), lit );
     old_value = object;
 }
 
@@ -1545,27 +1557,16 @@ typedef wif_T_trace<sc_dt::sc_lv_base> wif_sc_lv_trace;
 
 
 wif_trace_file::wif_trace_file(const char * name)
-: fp(0), trace_delta_cycles(false), wif_name_index(0),
-  previous_time_units_low(0), previous_time_units_high(0), previous_time(0.0),
-  traces(), initialized(false),
-  timescale_unit(sc_get_time_resolution().to_seconds()),
-  timescale_set_by_user(false)
-
-{
-    std::string file_name = name ;
-    file_name += ".awif";
-    fp = fopen(file_name.c_str(), "w");
-    if (!fp) {
-        std::string msg = 
-		std::string("Cannot write trace file '") + file_name +
-	    "'";
-	::std::cerr << "FATAL: " << msg << "\n";
-        exit(1);
-    }
-}
+  : sc_trace_file_base( name, "awif" )
+  , wif_name_index(0)
+  , previous_time_units_low(0)
+  , previous_time_units_high(0)
+  , previous_time(0.0)
+  , traces()
+{}
 
 
-void wif_trace_file::initialize()
+void wif_trace_file::do_initialize()
 {
     char buf[2000];
 
@@ -1593,26 +1594,14 @@ void wif_trace_file::initialize()
     else if(timescale_unit == 1e2)   std::sprintf(buf,"17");
     std::fprintf(fp,"header  %s \"%s\" ;\n\n", buf, sc_version());
 
-    //date:
-    time_t long_time;
-    time(&long_time);
-    struct tm* p_tm;
-    p_tm = localtime(&long_time);
-    strftime(buf, 199, "%b %d, %Y       %H:%M:%S", p_tm);
-    std::fprintf(fp, "comment \"ASCII WIF file produced on date:  %s\" ;\n", buf);
+    std::fprintf(fp, "comment \"ASCII WIF file produced on date:  %s\" ;\n"
+                , localtime_string().c_str());
  
     //version:
     std::fprintf(fp, "comment \"Created by %s\" ;\n", sc_version());
     //conversion info
     std::fprintf(fp, "comment \"Convert this file to binary WIF format using a2wif\" ;\n\n");
 
-
-    running_regression = ( getenv( "SYSTEMC_REGRESSION" ) != NULL );
-    // Don't print message if running regression
-    if( ! timescale_set_by_user && ! running_regression ) {
-	::std::cout << "WARNING: Default time step is used for WIF tracing." << ::std::endl;
-    }
-
     // Define the two types we need to represent bool and sc_logic
     std::fprintf(fp, "type scalar \"BIT\" enum '0', '1' ;\n");
     std::fprintf(fp, "type scalar \"MVL\" enum '0', '1', 'X', 'Z', '?' ;\n");
@@ -1652,18 +1641,12 @@ void wif_trace_file::initialize()
 
 #define DEFN_TRACE_METHOD(tp)                                                 \
 void                                                                          \
-wif_trace_file::trace( const tp& object_, const std::string& name_ )     \
+wif_trace_file::trace( const tp& object_, const std::string& name_ )          \
 {                                                                             \
-    if( initialized ) {                                                       \
-        put_error_message(                                                \
-	    "No traces can be added once simulation has started.\n"           \
-            "To add traces, create a new wif trace file.", false );           \
-    }                                                                         \
-    std::string temp_wif_name;                                           \
-    create_wif_name( &temp_wif_name );                                        \
-    traces.push_back( new wif_ ## tp ## _trace( object_,                      \
-						name_,                        \
-						temp_wif_name ) );            \
+    if( add_trace_check(name_) )                                              \
+        traces.push_back( new wif_ ## tp ## _trace( object_,                  \
+                                                    name_,                    \
+                                                    obtain_name() ) );        \
 }
 
 DEFN_TRACE_METHOD(bool)
@@ -1673,18 +1656,12 @@ DEFN_TRACE_METHOD(double)
 #undef DEFN_TRACE_METHOD
 #define DEFN_TRACE_METHOD(tp)                                                 \
 void                                                                          \
-wif_trace_file::trace(const sc_dt::tp& object_, const std::string& name_)\
+wif_trace_file::trace(const sc_dt::tp& object_, const std::string& name_)     \
 {                                                                             \
-    if( initialized ) {                                                       \
-        put_error_message(                                                \
-	    "No traces can be added once simulation has started.\n"               \
-            "To add traces, create a new wif trace file.", false );           \
-    }                                                                         \
-    std::string temp_wif_name;                                           \
-    create_wif_name( &temp_wif_name );                                        \
-    traces.push_back( new wif_ ## tp ## _trace( object_,                      \
-						name_,                        \
-						temp_wif_name ) );            \
+    if( add_trace_check(name_) )                                              \
+        traces.push_back( new wif_ ## tp ## _trace( object_,                  \
+                                                    name_,                    \
+                                                    obtain_name() ) );        \
 }
 
 DEFN_TRACE_METHOD(sc_bit)
@@ -1705,40 +1682,28 @@ DEFN_TRACE_METHOD(sc_fxnum_fast)
 
 #define DEFN_TRACE_METHOD_SIGNED(tp)                                          \
 void                                                                          \
-wif_trace_file::trace( const tp&        object_,                              \
-                       const std::string& name_,                         \
-                       int              width_ )                              \
+wif_trace_file::trace( const tp&          object_,                            \
+                       const std::string& name_,                              \
+                       int                width_ )                            \
 {                                                                             \
-    if( initialized ) {                                                       \
-        put_error_message(                                                \
-	    "No traces can be added once simulation has started.\n"           \
-            "To add traces, create a new wif trace file.", false );           \
-    }                                                                         \
-    std::string temp_wif_name;                                           \
-    create_wif_name( &temp_wif_name );                                        \
-    traces.push_back( new wif_signed_ ## tp ## _trace( object_,               \
-						       name_,                 \
-						       temp_wif_name,         \
-                                                       width_ ) );            \
+    if( add_trace_check(name_) )                                              \
+        traces.push_back( new wif_signed_ ## tp ## _trace( object_,           \
+                                                           name_,             \
+                                                           obtain_name(),     \
+                                                           width_ ) );        \
 }
 
 #define DEFN_TRACE_METHOD_UNSIGNED(tp)                                        \
 void                                                                          \
 wif_trace_file::trace( const unsigned tp& object_,                            \
-                       const std::string&   name_,                       \
+                       const std::string& name_,                              \
                        int                width_ )                            \
 {                                                                             \
-    if( initialized ) {                                                       \
-        put_error_message(                                                \
-	    "No traces can be added once simulation has started.\n"           \
-            "To add traces, create a new wif trace file.", false );           \
-    }                                                                         \
-    std::string temp_wif_name;                                           \
-    create_wif_name( &temp_wif_name );                                        \
-    traces.push_back( new wif_unsigned_ ## tp ## _trace( object_,             \
-						         name_,               \
-						         temp_wif_name,       \
-                                                         width_ ) );          \
+    if( add_trace_check(name_) )                                              \
+        traces.push_back( new wif_unsigned_ ## tp ## _trace( object_,         \
+                                                             name_,           \
+                                                             obtain_name(),   \
+                                                             width_ ) );      \
 }
 
 DEFN_TRACE_METHOD_SIGNED(char)
@@ -1757,21 +1722,15 @@ DEFN_TRACE_METHOD_UNSIGNED(long)
 
 #define DEFN_TRACE_METHOD_LONG_LONG(tp)                                       \
 void                                                                          \
-wif_trace_file::trace( const sc_dt::tp& object_,                              \
-                       const std::string&   name_,                       \
+wif_trace_file::trace( const sc_dt::tp&   object_,                            \
+                       const std::string& name_,                              \
                        int                width_ )                            \
 {                                                                             \
-    if( initialized ) {                                                       \
-        put_error_message(                                                \
-	    "No traces can be added once simulation has started.\n"           \
-            "To add traces, create a new wif trace file.", false );           \
-    }                                                                         \
-    std::string temp_wif_name;                                           \
-    create_wif_name( &temp_wif_name );                                        \
-    traces.push_back( new wif_ ## tp ## _trace( object_,                      \
-						name_,                        \
-						temp_wif_name,                \
-                                                width_ ) );                   \
+    if( add_trace_check(name_) )                                              \
+        traces.push_back( new wif_ ## tp ## _trace( object_,                  \
+                                                    name_,                    \
+                                                    obtain_name(),            \
+                                                    width_ ) );               \
 }
 
 DEFN_TRACE_METHOD_LONG_LONG(int64)
@@ -1783,17 +1742,11 @@ wif_trace_file::trace( const unsigned& object_,
 		       const std::string& name_,
 		       const char** enum_literals_ )
 {
-    if( initialized ) {
-        put_error_message(
-	    "No traces can be added once simulation has started.\n"
-	    "To add traces, create a new wif trace file.", false );
-    }
-    std::string temp_wif_name;
-    create_wif_name( &temp_wif_name );
-    traces.push_back( new wif_enum_trace( object_,
-					  name_,
-					  temp_wif_name,
-					  enum_literals_ ) );
+    if( add_trace_check(name_) )
+        traces.push_back( new wif_enum_trace( object_,
+                                              name_,
+                                              obtain_name(),
+                                              enum_literals_ ) );
 }
 
 void
@@ -1814,29 +1767,22 @@ wif_trace_file::trace( const sc_dt::sc_lv_base& object_,
 void
 wif_trace_file::write_comment(const std::string& comment)
 {
+    if(!fp) open_fp();
     //no newline in comments allowed
     std::fprintf(fp, "comment \"%s\" ;\n", comment.c_str());
 }
 
 
-void
-wif_trace_file::delta_cycles(bool flag)
-{
-    trace_delta_cycles = flag;
-}
-
 void
 wif_trace_file::cycle(bool this_is_a_delta_cycle)
 {
     unsigned now_units_high, now_units_low;
 
     // Trace delta cycles only when enabled
-    if (!trace_delta_cycles && this_is_a_delta_cycle) return;
+    if (!delta_cycles() && this_is_a_delta_cycle) return;
 
     // Check for initialization
-    if (!initialized) {
-        initialize();
-        initialized = true;
+    if( initialize() ) {
         return;
     };
 
@@ -1887,73 +1833,29 @@ wif_trace_file::cycle(bool this_is_a_delta_cycle)
     }
 }
 
-// Create a WIF name for a variable
+#if 0
 void
 wif_trace_file::create_wif_name(std::string* ptr_to_str)
 {
-    char buf[50];
-    std::sprintf(buf,"O%d", wif_name_index);
-    *ptr_to_str = buf; 
-    wif_name_index++;
+  obtain_name().swap(*ptr_to_str);
 }
+#endif
 
+// Create a WIF name for a variable
 std::string
-wif_trace_file::obtain_new_index()
+wif_trace_file::obtain_name()
 {
     char buf[32];
     std::sprintf( buf, "O%d", wif_name_index ++ );
-    return std::string( buf );
+    return buf;
 }
 
-// Set the time unit
-void wif_trace_file::set_time_unit(double v, sc_time_unit tu)
-{
-    if(initialized)
-    {        
-	put_error_message(
-	    "Trace timescale unit cannot be changed once tracing has begun.",
-	    false
-        );
-        std::cout << "To change the scale, create a new trace file." 
-		  << std::endl;
-        return;    
-    }
-
-    switch ( tu )
-    {
-      case SC_FS:  v = v * 1e-15; break;
-      case SC_PS:  v = v * 1e-12; break;
-      case SC_NS:  v = v * 1e-9;  break;
-      case SC_US:  v = v * 1e-6;  break;
-      case SC_MS:  v = v * 1e-3;  break;
-      case SC_SEC:                break;
-      default:                    
-	  	put_error_message("Unknown time unit specified ",true);
-		std::cout << tu << std::endl;
-		break;
-    }
-    timescale_unit = v;
-
-    // EMIT ADVISORY MESSAGE ABOUT CHANGE IN TIME SCALE:
-
-    char buf[200];
-    std::sprintf(buf,
-        "Note: VCD trace timescale unit is set by user to %e sec.\n",
-        timescale_unit);
-    ::std::cout << buf << ::std::flush;
-
-	timescale_set_by_user = true;
-}
-
-// Cleanup and close trace file
 wif_trace_file::~wif_trace_file()
 {
-    int i;
-    for (i = 0; i < (int)traces.size(); i++) {
+    for( int i = 0; i < (int)traces.size(); i++ ) {
         wif_trace* t = traces[i];
         delete t;
     }
-    fclose(fp);
 }
 
 // Map sc_logic values to values understandable by WIF
@@ -1986,36 +1888,21 @@ map_sc_logic_state_to_wif_state(char in_char)
     return out_char;
 }
 
-
-#if 0
-// no output should be done directly to ::std::cout, cerr, etc.
-void
-put_error_message(const char* msg, bool just_warning)
-{
-    if(just_warning){
-	::std::cout << "WIF Trace Warning:\n" << msg << "\n" << ::std::endl;
-    }
-    else{
-	::std::cout << "WIF Trace ERROR:\n" << msg << "\n" << ::std::endl;
-    }
-}
-#endif // 0
+// ----------------------------------------------------------------------------
 
 // Create the trace file
 sc_trace_file*
 sc_create_wif_trace_file(const char * name)
 {
-    sc_trace_file *tf;
-
-    tf = new wif_trace_file(name);
-    sc_get_curr_simcontext()->add_trace_file(tf);
+    sc_trace_file *tf = new wif_trace_file(name);
     return tf;
 }
 
+
 void
 sc_close_wif_trace_file( sc_trace_file* tf )
 {
-    wif_trace_file* wif_tf = (wif_trace_file*)tf;
+    wif_trace_file* wif_tf = static_cast<wif_trace_file*>(tf);
     delete wif_tf;
 }
 
