@@ -1,14 +1,14 @@
 /*****************************************************************************
 
   The following code is derived, directly or indirectly, from the SystemC
-  source code Copyright (c) 1996-2011 by all Contributors.
+  source code Copyright (c) 1996-2014 by all Contributors.
   All Rights reserved.
 
   The contents of this file are subject to the restrictions and limitations
-  set forth in the SystemC Open Source License Version 3.0 (the "License");
+  set forth in the SystemC Open Source License (the "License");
   You may not use this file except in compliance with such restrictions and
   limitations. You may obtain instructions on how to receive a copy of the
-  License at http://www.systemc.org/. Software distributed by Contributors
+  License at http://www.accellera.org/. Software distributed by Contributors
   under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
   ANY KIND, either express or implied. See the License for the specific
   language governing rights and limitations under the License.
@@ -26,7 +26,7 @@
 
 #include <assert.h>
 #include <ctype.h>
-#include <stdio.h>
+#include <cstdio>
 #include <stdarg.h>
 #include <string.h>
 
@@ -36,52 +36,13 @@
 
 namespace sc_dt {
 
-inline int
+inline static int
 sc_roundup( int n, int m )
 {
     return ((n - 1) / m + 1) * m;
 }
 
 
-// ----------------------------------------------------------------------------
-//  ENUM : sc_numrep
-//
-//  Enumeration of number representations for character string conversion.
-// ----------------------------------------------------------------------------
-
-const std::string
-to_string( sc_numrep numrep )
-{
-    switch( numrep )
-    {
-        case SC_DEC:
-	    return std::string( "SC_DEC" );
-        case SC_BIN:
-	    return std::string( "SC_BIN" );
-        case SC_BIN_US:
-	    return std::string( "SC_BIN_US" );
-        case SC_BIN_SM:
-	    return std::string( "SC_BIN_SM" );
-        case SC_OCT:
-	    return std::string( "SC_OCT" );
-        case SC_OCT_US:
-	    return std::string( "SC_OCT_US" );
-        case SC_OCT_SM:
-	    return std::string( "SC_OCT_SM" );
-        case SC_HEX:
-	    return std::string( "SC_HEX" );
-        case SC_HEX_US:
-	    return std::string( "SC_HEX_US" );
-        case SC_HEX_SM:
-	    return std::string( "SC_HEX_SM" );
-        case SC_CSD:
-	    return std::string( "SC_CSD" );
-	default:
-	    return std::string( "unknown" );
-    }
-}
-
-
 // ----------------------------------------------------------------------------
 //  CLASS : sc_string_rep
 //
@@ -402,6 +363,13 @@ sc_string_old::set( int i, char c )
     rep->str[i] = c;
 }
 
+#if defined(_MSC_VER)
+   // Windows provides safer implementation
+#  define sc_vsnprintf _vsnprintf
+#else
+#  define sc_vsnprintf vsnprintf
+#endif
+
 sc_string_old sc_string_old::to_string(const char* format, ...)
 {
    va_list argptr;
@@ -410,29 +378,18 @@ sc_string_old sc_string_old::to_string(const char* format, ...)
    buffer[1023]=000;
 
    va_start(argptr, format);
-#if defined(WIN32)
-   // Windows provides safer implementation
-#if defined(_MSC_VER)
-   int cnt = _vsnprintf(buffer, 1024, format, argptr);
-#else
-   int cnt = vsnprintf(buffer, 1024, format, argptr);
-#endif
+   int cnt = sc_vsnprintf(buffer, 1024, format, argptr);
    if(cnt>1023) // string too long
    {
      int buf_size = 1024;
      const int max_size = 65000;
-     char* buf; // dynamic buffer
+     char* buf = 0; // dynamic string buffer
      do
      {
+       delete[] buf;
        buf_size*=2;
        buf = new char[buf_size];
-#if defined(_MSC_VER)
-       cnt = _vsnprintf(buffer, buf_size, format, argptr);
-#else
-       cnt = vsnprintf(buffer, buf_size, format, argptr);
-#endif
-       if(buf_size<max_size && cnt>=buf_size)
-         delete[] buf;
+       cnt = sc_vsnprintf(buf, buf_size, format, argptr);
      }
      while( buf_size<max_size && cnt>=buf_size);
      if(cnt>=buf_size)
@@ -446,20 +403,7 @@ sc_string_old sc_string_old::to_string(const char* format, ...)
    }
    else
      result = buffer;
-#else
-   try {
-     // this may end up in a core dump
-     // if we are lucky we can catch exception
-     vsprintf(buffer, format, argptr);
-   }
-   catch(...)
-   {
-     SC_REPORT_WARNING( sc_core::SC_ID_STRING_TOO_LONG_,
-			"program may become unstable" );
-   }
-   buffer[1023]=000; // in case it's longer
-   result = buffer;
-#endif
+
    va_end(argptr);
 
    return result;
